package com.barrelsofdata.examples.beam.streaming.sensor.config;

import org.apache.beam.sdk.options.Default;
import org.apache.beam.sdk.options.Description;
import org.apache.beam.sdk.options.PipelineOptions;
import org.apache.beam.sdk.options.Validation;

public interface SensorPipelineOptions extends PipelineOptions {

    @Description("Comma separated list of kafka brokers (host:port)")
    @Validation.Required
    String getKafkaBrokers();
    void setKafkaBrokers(String kafkaBrokers);

    @Description("Kafka topic to read from")
    @Validation.Required
    String getKafkaTopic();
    void setKafkaTopic(String kafkaTopic);

    @Description("Kafka consumer group id to read from")
    @Validation.Required
    String getKafkaConsumerGroupId();
    void setKafkaConsumerGroupId(String kafkaConsumerGroupId);

    @Description("Set kafka auto offset reset to earliest (default: latest)")
    @Default.Boolean(false)
    boolean isResetToEarliest();
    void setResetToEarliest(boolean resetToEarliest);

    @Description("Kafka max read duration in minutes (Only used for testing)")
    Integer getKafkaMaxReadDurationMinutes();
    void setKafkaMaxReadDurationMinutes(Integer kafkaMaxReadDurationMinutes);

    @Description("Sql driver to be used for jdbc")
    @Validation.Required
    String getSqlDriver();
    void setSqlDriver(String sqlDriver);

    @Description("Target jdbc url to connect to")
    @Validation.Required
    String getJdbcUrl();
    void setJdbcUrl(String jdbcUrl);

    @Description("Target table to write to")
    @Validation.Required
    String getTable();
    void setTable(String table);

    @Description("Username to access the jdbc data target")
    @Validation.Required
    String getUsername();
    void setUsername(String username);

    @Description("Password of the provided user")
    @Validation.Required
    String getPassword();
    void setPassword(String table);
}
