package com.barrelsofdata.examples.beam.streaming.sensor.function;

import com.barrelsofdata.examples.beam.streaming.sensor.model.FailedEvent;
import org.apache.beam.sdk.transforms.SimpleFunction;
import org.apache.beam.sdk.transforms.WithFailures;

import java.util.Objects;

/**
 * Builder that builds the {@link FailedEvent}
 */
public class FailedEventBuilder<T> extends SimpleFunction<WithFailures.ExceptionElement<T>, FailedEvent> {

    private final String step;

    private FailedEventBuilder(String step) {
        this.step = step;
    }

    @Override
    public FailedEvent apply(WithFailures.ExceptionElement<T> input) {
        if(input != null)
            return new FailedEvent(step, Objects.toString(input.element(), null), input.exception());
        return new FailedEvent(step, null, null);
    }

    public static <T> FailedEventBuilder<T> of(String step) {
        return new FailedEventBuilder<>(step);
    }
}
