package com.barrelsofdata.examples.beam.streaming.sensor.target;

import com.barrelsofdata.examples.beam.streaming.sensor.config.Steps;
import com.barrelsofdata.examples.beam.streaming.sensor.model.ComputedEvent;
import org.apache.beam.sdk.io.jdbc.JdbcIO;
import org.apache.beam.sdk.transforms.PTransform;
import org.apache.beam.sdk.values.PCollection;
import org.apache.beam.sdk.values.PDone;

import java.sql.Date;
import java.sql.Timestamp;

/**
 * PTransform that writes the {@link ComputedEvent} into a target table through a jdbc connection
 */
public class JDBCTarget extends PTransform<PCollection<ComputedEvent>, PDone> {

    private final String driver;
    private final String jdbcUrl;
    private final String table;
    private final String username;
    private final String password;

    public JDBCTarget(String driver, String jdbcUrl, String table, String username, String password) {
        this.driver = driver;
        this.jdbcUrl = jdbcUrl;
        this.table = table;
        this.username = username;
        this.password = password;
    }

    @Override
    public PDone expand(PCollection<ComputedEvent> input) {
        return input.apply(Steps.JDBC_WRITE.getStep(), JdbcIO.<ComputedEvent>write()
                        .withDataSourceConfiguration(JdbcIO.DataSourceConfiguration
                                .create(driver, jdbcUrl)
                                .withUsername(username)
                                .withPassword(password))
                        .withStatement("INSERT INTO %s VALUES(?, ?, ?)".formatted(table)) // Use Merge if you want to avoid duplicates
                        .withPreparedStatementSetter((JdbcIO.PreparedStatementSetter<ComputedEvent>) (element, query) -> {
                            query.setString(1, element.id());
                            query.setTimestamp(2, new Timestamp(element.ts()));
                            query.setDouble(3, element.value());
                        }));
    }
}
