package com.barrelsofdata.examples.beam.streaming.sensor.target;

import com.barrelsofdata.examples.beam.streaming.sensor.model.ComputedEvent;
import org.apache.beam.sdk.coders.SerializableCoder;
import org.apache.beam.sdk.testing.TestPipeline;
import org.apache.beam.sdk.transforms.Create;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.sql.*;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

class JDBCTargetTest {

    private final static String driver = "org.h2.Driver";
    private final static String jdbcUrl = "jdbc:h2:mem:testDb";
    private final static String table = "testTable";
    private final static String username = "";
    private final static String password = "";
    private static Connection con;
    private static Statement statement;

    @BeforeAll
    static void initDb() throws SQLException {
        con = DriverManager.getConnection(jdbcUrl, username, password);
        statement = con.createStatement();
        statement.executeUpdate("""
                CREATE TABLE %s (id VARCHAR(255), ts TIMESTAMP, computed DOUBLE)""".formatted(table));
    }

    @AfterAll
    static void teardown() throws SQLException {
        con.close();
    }

    @Test
    void testExpectedDataWasWrittenToTable() throws SQLException {
        String id1 = "testId1";
        String id2 = "testId2";
        Long ts = Instant.now().toEpochMilli();
        Double value = 10.0;
        List<ComputedEvent> events = List.of(
                new ComputedEvent(id1, ts, value),
                new ComputedEvent(id2, ts, value));

        JDBCTarget jdbcTarget = new JDBCTarget(driver, jdbcUrl, table, username, password);

        TestPipeline pipeline = TestPipeline.create().enableAbandonedNodeEnforcement(false);

        pipeline
                .apply(Create.of(events))
                .setCoder(SerializableCoder.of(ComputedEvent.class))
                .apply(jdbcTarget);

        pipeline.run()
                .waitUntilFinish();

        ResultSet results = statement.executeQuery("SELECT * FROM %s".formatted(table));
        List<ComputedEvent> dbResults = new ArrayList<>();
        while(results.next())
            dbResults.add(new ComputedEvent(results.getString("id"), results.getTimestamp("ts").getTime(), results.getDouble("computed")));

        assertTrue(dbResults.containsAll(events));
    }

}