package com.barrelsofdata.examples.beam.streaming.sensor.transform;

import com.barrelsofdata.examples.beam.streaming.sensor.model.RawEvent;
import org.apache.beam.sdk.testing.PAssert;
import org.apache.beam.sdk.testing.TestPipeline;
import org.apache.beam.sdk.transforms.Create;
import org.apache.beam.sdk.transforms.DoFn;
import org.apache.beam.sdk.transforms.ParDo;
import org.apache.beam.sdk.values.KV;
import org.apache.beam.sdk.values.PCollection;
import org.joda.time.Instant;
import org.junit.jupiter.api.Test;

class ParseWithTimestampIT {

    @Test
    void testThatDataIsParsedAndTimestampAttached() {
        Long ts = 1698489173051L;
        KV<String, String> inputEvent = KV.of("testId1", """
                {"ts":%s,"value":10.0}""".formatted(ts));

        ParseWithTimestamp parseWithTimestamp = new ParseWithTimestamp();

        TestPipeline pipeline = TestPipeline.create().enableAbandonedNodeEnforcement(false);

        PCollection<Long> parsed = pipeline
                .apply(Create.of(inputEvent))
                .apply(parseWithTimestamp)
                .apply(ParDo.of(new BeamTimestampGrabber()));

        PAssert.that(parsed).containsInAnyOrder(ts);
        pipeline.run();
    }

    private static class BeamTimestampGrabber extends DoFn<RawEvent, Long> {
        @ProcessElement
        public void process(ProcessContext context, @Timestamp Instant timestamp) {
            context.output(timestamp.getMillis());
        }
    }

}