package com.barrelsofdata.examples.beam.streaming.sensor.util;

import com.barrelsofdata.examples.beam.streaming.sensor.config.SensorPipelineOptions;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;

import static org.junit.jupiter.api.Assertions.*;

class PipelineOptionsBuilderTest {

    @ParameterizedTest
    @CsvSource(value = {
            "DirectRunner|localhost:9092,localhost:9093|test_topic|test_cg_1|true|org.h2.driver|jdbc:h2:mem:testdb|testTable|user|pass",
            "DirectRunner|127.0.0.1|test_topic|test_cg_2|false|org.h2.driver|jdbc:h2:mem:testdb|testTable|user|pass"
    }, delimiter = '|')
    void testOptionsParsing(String runner, String kafkaBrokers, String kafkaTopic, String kafkaConsumerGroupId, Boolean resetToEarliest, String sqlDriver, String jdbcUrl, String table, String username, String password) {
        String[] args = {
                "--runner=%s".formatted(runner),
                "--kafkaBrokers=%s".formatted(kafkaBrokers),
                "--kafkaTopic=%s".formatted(kafkaTopic),
                "--kafkaConsumerGroupId=%s".formatted(kafkaConsumerGroupId),
                "--sqlDriver=%s".formatted(sqlDriver),
                "--jdbcUrl=%s".formatted(jdbcUrl),
                "--table=%s".formatted(table),
                "--username=%s".formatted(username),
                "--password=%s".formatted(password),
                resetToEarliest ? "--resetToEarliest" : ""
        };
        SensorPipelineOptions options = PipelineOptionsBuilder.from(args, SensorPipelineOptions.class);

        assertEquals(runner, options.getRunner().getSimpleName());
        assertEquals(kafkaBrokers, options.getKafkaBrokers());
        assertEquals(kafkaTopic, options.getKafkaTopic());
        assertEquals(kafkaConsumerGroupId, options.getKafkaConsumerGroupId());
        assertEquals(sqlDriver, options.getSqlDriver());
        assertEquals(jdbcUrl, options.getJdbcUrl());
        assertEquals(table, options.getTable());
        assertEquals(username, options.getUsername());
        assertEquals(password, options.getPassword());
    }

}